"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.wrapKey = exports.unwrapKey = exports.unauthenticInputException = exports.invalidWrapInputException = exports.invalidUnwrapInputException = void 0;
const buffer_1 = require("buffer/");
const aeskeywrap_1 = require("./aeskeywrap");
const unauthenticInputException = new Error('Unauthentic data. Wrong kek?');
exports.unauthenticInputException = unauthenticInputException;
const invalidWrapInputException = new Error('Invalid data length(s). kek must be 16, 24 or 32 bytes, key must be same length.');
exports.invalidWrapInputException = invalidWrapInputException;
const invalidUnwrapInputException = new Error('Invalid data length(s). kek must be 16, 24 or 32 bytes, wrappedKey must be 8 bytes longer.');
exports.invalidUnwrapInputException = invalidUnwrapInputException;
/**
 * Wrapps (respectively encrypts) a key with a kek (key-encryption key), using aes-key-wrap.
 * Key and kek, both must be 16, 24, or 32 bytes long
 * @param key the key to wrap (Unit8Array)
 * @param kek the kek (Unit8Array)
 * @throws an Error if key and/or kek length is invalid
 * @returns the wrapped key (Unit8Array)
 */
const wrapKey = function (key, kek) {
    const keyBuffer = buffer_1.Buffer.from(key);
    const kekBuffer = buffer_1.Buffer.from(kek);
    try {
        return new Uint8Array((0, aeskeywrap_1.aesWrapKey)(keyBuffer, kekBuffer));
    }
    catch {
        throw invalidWrapInputException;
    }
};
exports.wrapKey = wrapKey;
/**
 * Unwrapps (respectively decrypts) a wrapped key with a kek (key-encryption key), using aes-key-unwrap.
 * Kek must be 16, 24, or 32 bytes long, wrapped key must be 8 bytes longer.
 * @param wrappedKey the wrapped key to unwrap (Unit8Array)
 * @param kek the kek (Unit8Array)
 * @throws an Error if wrappedKey and/or kek length is invalid
 * @throws an Error on unauthentic data (Message: 'Unauthentic data. Wrong kek?')
 * @returns the unwrapped key (Unit8Array)
 */
const unwrapKey = function (wrappedKey, kek) {
    const wrappedKeyBuffer = buffer_1.Buffer.from(wrappedKey);
    const kekBuffer = buffer_1.Buffer.from(kek);
    let keyBuffer;
    try {
        keyBuffer = (0, aeskeywrap_1.aesUnwrapKey)(wrappedKeyBuffer, kekBuffer);
    }
    catch {
        throw invalidUnwrapInputException;
    }
    if (!keyBuffer) {
        throw unauthenticInputException;
    }
    return new Uint8Array(keyBuffer);
};
exports.unwrapKey = unwrapKey;
