"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assertthat_1 = require("assertthat");
const buffer_1 = require("buffer/");
const crypto_js_1 = __importDefault(require("crypto-js"));
const aeskeywrap_1 = require("../../lib/aeskeywrap");
const wrapping_1 = require("../../lib/wrapping");
const getRandomUint8Array = (sizeInBytes) => {
    const randomWordArray = crypto_js_1.default.lib.WordArray.random(sizeInBytes);
    return new Uint8Array((0, aeskeywrap_1.wordArrayToBuffer)(randomWordArray));
};
suite('wrapping', () => {
    // Official test vecors
    const testVectors = [
        // https://datatracker.ietf.org/doc/html/rfc3394#section-4.1
        {
            length: 16,
            key: new Uint8Array(buffer_1.Buffer.from('00112233445566778899AABBCCDDEEFF', 'hex')),
            kek: new Uint8Array(buffer_1.Buffer.from('000102030405060708090A0B0C0D0E0F', 'hex')),
            wrappedKey: new Uint8Array(buffer_1.Buffer.from('1FA68B0A8112B447AEF34BD8FB5A7B829D3E862371D2CFE5', 'hex'))
        },
        // https://datatracker.ietf.org/doc/html/rfc3394#section-4.4
        {
            length: 24,
            key: new Uint8Array(buffer_1.Buffer.from('00112233445566778899AABBCCDDEEFF0001020304050607', 'hex')),
            kek: new Uint8Array(buffer_1.Buffer.from('000102030405060708090A0B0C0D0E0F1011121314151617', 'hex')),
            wrappedKey: new Uint8Array(buffer_1.Buffer.from('031D33264E15D33268F24EC260743EDCE1C6C7DDEE725A936BA814915C6762D2', 'hex'))
        },
        // https://datatracker.ietf.org/doc/html/rfc3394#section-4.6
        {
            length: 32,
            key: new Uint8Array(buffer_1.Buffer.from('00112233445566778899AABBCCDDEEFF000102030405060708090A0B0C0D0E0F', 'hex')),
            kek: new Uint8Array(buffer_1.Buffer.from('000102030405060708090A0B0C0D0E0F101112131415161718191A1B1C1D1E1F', 'hex')),
            wrappedKey: new Uint8Array(buffer_1.Buffer.from('28C9F404C4B810F4CBCCB35CFB87F8263F5786E2D80ED326CBC7F0E71A99F43BFB988B9B7A02DD21', 'hex'))
        }
    ];
    suite('using test vectors', () => {
        for (const { length, key, kek, wrappedKey } of testVectors) {
            suite(`${length * 8} bit`, () => {
                test('wrap returns correct wrapped key.', async () => {
                    const actualWrappedKey = (0, wrapping_1.wrapKey)(key, kek);
                    assertthat_1.assert.that(actualWrappedKey).is.equalTo(wrappedKey);
                });
                test('unwrap returns correct unwrapped key.', async () => {
                    const actualUnwrappedKey = (0, wrapping_1.unwrapKey)(wrappedKey, kek);
                    assertthat_1.assert.that(actualUnwrappedKey).is.equalTo(key);
                });
            });
        }
    });
    suite('randomly forth and back', () => {
        for (const { length } of testVectors) {
            test(`${length * 8} bit`, async () => {
                const key = getRandomUint8Array(length);
                const kek = getRandomUint8Array(length);
                const wrappedKey = (0, wrapping_1.wrapKey)(key, kek);
                const keyBack = (0, wrapping_1.unwrapKey)(wrappedKey, kek);
                assertthat_1.assert.that(keyBack).is.equalTo(key);
            });
        }
    });
    suite('error cases', () => {
        test('throws unauthenticInputException when wrappedKey was tampered with.', async () => {
            const key = getRandomUint8Array(32);
            const kek = getRandomUint8Array(32);
            const wrappedKey = (0, wrapping_1.wrapKey)(key, kek);
            // Change first byte
            // eslint-disable-next-line no-bitwise
            wrappedKey.set([(wrappedKey.at(0) ?? 0) ^ 1]);
            assertthat_1.assert.that(() => {
                (0, wrapping_1.unwrapKey)(wrappedKey, kek);
            }).is.throwing(wrapping_1.unauthenticInputException.message);
        });
        test('throws unauthenticInputException on invalid kek.', async () => {
            const key = getRandomUint8Array(32);
            const kek = getRandomUint8Array(32);
            const wrappedKey = (0, wrapping_1.wrapKey)(key, kek);
            // Change first byte
            // eslint-disable-next-line no-bitwise
            kek.set([(kek.at(0) ?? 0) ^ 1]);
            assertthat_1.assert.that(() => {
                (0, wrapping_1.unwrapKey)(wrappedKey, kek);
            }).is.throwing(wrapping_1.unauthenticInputException.message);
        });
        test('throws exception on invalid kek length (wrap).', async () => {
            const key = getRandomUint8Array(32);
            const kek = getRandomUint8Array(20);
            assertthat_1.assert.that(() => {
                (0, wrapping_1.wrapKey)(key, kek);
            }).is.throwing(wrapping_1.invalidWrapInputException.message);
        });
        test('throws exception on invalid kek length (unwrap).', async () => {
            const wrappedKey = getRandomUint8Array(40);
            const kek = getRandomUint8Array(20);
            assertthat_1.assert.that(() => {
                (0, wrapping_1.unwrapKey)(wrappedKey, kek);
            }).is.throwing(wrapping_1.invalidUnwrapInputException.message);
        });
        test('throws exception on invalid key length for kek.', async () => {
            const key = getRandomUint8Array(24);
            const kek = getRandomUint8Array(32);
            assertthat_1.assert.that(() => {
                (0, wrapping_1.wrapKey)(key, kek);
            }).is.throwing(wrapping_1.invalidWrapInputException.message);
        });
        test('throws exception on invalid wrappedKey length for kek.', async () => {
            const wrappedKey = getRandomUint8Array(32);
            const kek = getRandomUint8Array(32);
            assertthat_1.assert.that(() => {
                (0, wrapping_1.unwrapKey)(wrappedKey, kek);
            }).is.throwing(wrapping_1.invalidUnwrapInputException.message);
        });
    });
});
